﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO.Ports;
using System.Reflection;
using System.Diagnostics;

namespace Terminal
{
    public partial class Form1 : Form
    {
        private SerialPort mySerialPort;
        private string prijateData = string.Empty;
        private mod recMod = mod.ASCI;
        private mod transMod = mod.ASCI;
        private string separator = " ";
        enum mod
        {
            ASCI = 0,
            HEX,
            DEC,
            BINARY
        }

        public Form1()
        {
            InitializeComponent();
            this.Text = "Terminal " + "v." + Assembly.GetExecutingAssembly().GetName().Version.ToString();
            this.initCombobox();
            this.enabledCombobox(true);
            this.mySerialPort = new SerialPort();
            this.mySerialPort.DataReceived += new SerialDataReceivedEventHandler(mySerialPort_DataReceived);
            this.mySerialPort.PinChanged += new SerialPinChangedEventHandler(mySerialPort_PinChanged);
        }

        void mySerialPort_PinChanged(object sender, SerialPinChangedEventArgs e)
        {
            this.button5_CTS.BackColor = this.mySerialPort.CtsHolding ? Color.Green : Color.Silver;
            this.button_DSR.BackColor = this.mySerialPort.DsrHolding ? Color.Green : Color.Silver;
            this.button7_CD.BackColor = this.mySerialPort.CDHolding ? Color.Green : Color.Silver;
        }
        delegate void SetControlValueCallback(Control oControl, string propName, object propValue);
        private void SetControlPropertyValue(Control oControl, string propName, object propValue)
        {
            if (oControl.InvokeRequired)
            {
                SetControlValueCallback d = new SetControlValueCallback(SetControlPropertyValue);
                oControl.Invoke(d, new object[] { oControl, propName, propValue });
            }
            else
            {
                Type t = oControl.GetType();
                PropertyInfo[] props = t.GetProperties();
                foreach (PropertyInfo p in props)
                {
                    if (p.Name.ToUpper() == propName.ToUpper())
                    {
                        p.SetValue(oControl, propValue, null);
                    }
                }
            }
        }
        void mySerialPort_DataReceived(object sender, SerialDataReceivedEventArgs e)
        {
            try
            {
                byte[] buffer = new byte[0];

                while (mySerialPort.BytesToRead > 0)
                {
                    buffer = new byte[mySerialPort.BytesToRead];
                    this.mySerialPort.Read(buffer, 0, mySerialPort.BytesToRead);
                }

                if (this.recMod == mod.ASCI)
                {
                    this.prijateData += Encoding.ASCII.GetString(buffer);
                }
                if (this.recMod == mod.HEX)
                {
                    foreach (byte s in buffer)
                    {
                        this.prijateData += s.ToString("X2") + this.separator;
                    }
                }
                if (this.recMod == mod.DEC)
                {
                    foreach (byte s in buffer)
                    {
                        this.prijateData += s.ToString() + this.separator;
                    }
                }
                if (this.recMod == mod.BINARY)
                {
                    foreach (byte s in buffer)
                    {
                        this.prijateData += Convert.ToString(s, 2).PadLeft(8, '0') + this.separator;
                    }
                }

                // this.prijateData += Encoding.ASCII.GetString(buffer);
                this.SetControlPropertyValue(this.richTextBox_rec, "Text", this.prijateData);
            }
            catch (Exception ex1)
            {
                MessageBox.Show(ex1.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void initCombobox()
        {
            //parity
            foreach (string s in Enum.GetNames(typeof(Parity)))
            {
                this.comboBox_parity.Items.Add(s);
            }
            //stop bits
            foreach (string s in Enum.GetNames(typeof(StopBits)))
            {
                this.comboBox_stop.Items.Add(s);
            }
            //data bits
            this.comboBox_data.Items.AddRange(new string[] { "5", "6", "7", "8" });
            //baud rate
            this.comboBox_baud.Items.AddRange(
                new string[] {
                "600",
                "1200",
                "2400",
                "4800",
                "9600",
                "14400",
                "19200",
                "28800",
                "38400",
                "56000",
                "57600",
                "115200",
                "128000",
                "256000", }
                );
            //COM port list
            this.comboBox_ports.Items.AddRange(SerialPort.GetPortNames());

            this.comboBox_CRC.Items.AddRange(new string[] { "None", "CRC16"});

            this.comboBox_ports.SelectedIndex = 0;
            this.comboBox_baud.SelectedIndex = 4;
            this.comboBox_data.SelectedIndex = 3;
            this.comboBox_parity.SelectedIndex = 0;
            this.comboBox_ports.SelectedIndex = 0;
            this.comboBox_rec.SelectedIndex = 0;
            this.comboBox_stop.SelectedIndex = 1;
            this.comboBox_trans.SelectedIndex = 0;
            this.comboBox_sep.SelectedIndex = 0;
            this.comboBox_CRC.SelectedIndex = 0;
        }
        private void enabledCombobox(bool enable)
        {
            this.comboBox_baud.Enabled = enable;
            this.comboBox_data.Enabled = enable;
            this.comboBox_parity.Enabled = enable;
            this.comboBox_ports.Enabled = enable;
            this.comboBox_rec.Enabled = enable;
            this.comboBox_stop.Enabled = enable;
            this.comboBox_trans.Enabled = enable;
            this.button_send.Enabled = !enable;
        }

        private void button1_Click(object sender, EventArgs e)
        {
            try
            {
                if (this.mySerialPort.IsOpen == false)
                {
                    
                    this.mySerialPort.PortName = this.comboBox_ports.Text;
                    this.mySerialPort.BaudRate = Convert.ToInt32(this.comboBox_baud.Text);
                    this.mySerialPort.DataBits = Convert.ToInt32(this.comboBox_data.Text);
                    this.mySerialPort.Parity = (Parity)Enum.Parse(typeof(Parity), this.comboBox_parity.Text);
                    this.mySerialPort.StopBits = (StopBits)Enum.Parse(typeof(StopBits), this.comboBox_stop.Text);
                    
                    this.mySerialPort.Open();
                    this.enabledCombobox(false);
                    
                    this.button1.Text = "Disconnect";
                }
                else
                {
                    this.mySerialPort.Close();
                    this.enabledCombobox(true);
                    this.button1.Text = "Connect";
                }
            }
            catch (Exception ex1)
            {
                MessageBox.Show(ex1.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void button2_Click(object sender, EventArgs e)
        {
            this.richTextBox_rec.Clear();
            this.prijateData = string.Empty;
        }

        private void button3_Click(object sender, EventArgs e)
        {
            this.richTextBox_trans.Clear();
        }

        private void button5_Click(object sender, EventArgs e)
        {
            try
            {
                if (this.transMod == mod.ASCI)
                {
                    this.mySerialPort.WriteLine(this.richTextBox_trans.Text);
                }
                if (this.transMod == mod.HEX)
                {
                    byte[] data;
                    int i = 0;
                    string[] dataString = this.richTextBox_trans.Text.Split(new string[] { this.comboBox_sep.Text }, StringSplitOptions.RemoveEmptyEntries);
                    data = new byte[dataString.Length];
                    foreach (string d1 in dataString)
                    {
                        d1.Replace("0x", "");
                        d1.Replace("0X", "");
                        data[i++] = Convert.ToByte(d1, 16);
                    }

                    this.mySerialPort.Write(data, 0, data.Length);
                    if (this.comboBox_CRC.Text == "CRC16")
                    {
                        byte uchCRCHi, uchCRCLo;
                        ClassCRC.CRC16(data, out uchCRCHi, out uchCRCLo);
                        this.mySerialPort.Write(new byte[] { uchCRCHi }, 0, 1);
                        this.mySerialPort.Write(new byte[] { uchCRCLo }, 0, 1);
                        this.textBox1.Text = "0x" + uchCRCHi.ToString("X2") + uchCRCLo.ToString("X2");
                    }
                }
                if (this.transMod == mod.DEC)
                {
                    byte[] data;
                    int i = 0;
                    string[] dataString = this.richTextBox_trans.Text.Split(new string[] { this.comboBox_sep.Text }, StringSplitOptions.RemoveEmptyEntries);
                    data = new byte[dataString.Length];
                    foreach (string d1 in dataString)
                    {
                        data[i++] = Convert.ToByte(d1, 10);
                    }

                    this.mySerialPort.Write(data, 0, data.Length);
                }
                if (this.transMod == mod.BINARY)
                {
                    byte[] data;
                    int i = 0;
                    string[] dataString = this.richTextBox_trans.Text.Split(new string[] { this.comboBox_sep.Text }, StringSplitOptions.RemoveEmptyEntries);
                    data = new byte[dataString.Length];
                    foreach (string d1 in dataString)
                    {
                        data[i++] = Convert.ToByte(d1, 2);
                    }

                    this.mySerialPort.Write(data, 0, data.Length);
                }
            }
            catch (Exception ex1)
            {
                MessageBox.Show(ex1.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void comboBox_trans_SelectedIndexChanged(object sender, EventArgs e)
        {
            this.comboBox_sep.Enabled = false;
            this.comboBox_CRC.Enabled = false;
            //trans
            this.separator = this.comboBox_sep.Text;
            if (this.comboBox_trans.Text.CompareTo("ASCI") == 0)
            {
                this.transMod = mod.ASCI;

            }
            if (this.comboBox_trans.Text.CompareTo("HEX") == 0)
            {
                this.comboBox_sep.Enabled = true;
                this.comboBox_CRC.Enabled = true;
                this.transMod = mod.HEX;
            }
            if (this.comboBox_trans.Text.CompareTo("DEC") == 0)
            {
                this.comboBox_sep.Enabled = true;
                this.transMod = mod.DEC;
            }
            if (this.comboBox_trans.Text.CompareTo("BINARY") == 0)
            {
                this.comboBox_sep.Enabled = true;
                this.transMod = mod.BINARY;
            }
            //rec
            if (this.comboBox_rec.Text.CompareTo("ASCI") == 0)
            {
                this.recMod = mod.ASCI;
            }
            if (this.comboBox_rec.Text.CompareTo("HEX") == 0)
            {
                this.comboBox_sep.Enabled = true;
                this.recMod = mod.HEX;
            }
            if (this.comboBox_rec.Text.CompareTo("DEC") == 0)
            {
                this.comboBox_sep.Enabled = true;
                this.recMod = mod.DEC;
            }
            if (this.comboBox_rec.Text.CompareTo("BINARY") == 0)
            {
                this.comboBox_sep.Enabled = true;
                this.recMod = mod.BINARY;
            }

        }


        private void Form1_Click(object sender, EventArgs e)
        {
            Process.Start(@"http://www.projectik.eu");
        }

        private void button8_DTR_Click(object sender, EventArgs e)
        {
            this.mySerialPort.DtrEnable = !this.mySerialPort.DtrEnable;
            this.button8_DTR.BackColor = this.mySerialPort.DtrEnable ? Color.Green : Color.Silver;
        }

        private void button4_RTS_Click(object sender, EventArgs e)
        {
            this.mySerialPort.RtsEnable = !this.mySerialPort.RtsEnable;
            this.button4_RTS.BackColor = this.mySerialPort.RtsEnable ? Color.Green : Color.Silver;
        }

        private void button4_Click(object sender, EventArgs e)
        {
            byte[] tempData = new byte[] { 112, 114, 111, 106, 101, 99, 116, 105, 107, 46, 101, 117 };//projectik.eu
            byte uchCRCHi, uchCRCLo;
            ClassCRC.CRC16(tempData, out uchCRCHi, out uchCRCLo);

            string myCRC16 = "0x" + uchCRCHi.ToString("X2") + uchCRCLo.ToString("X2");
                    

        }
    }
}
